"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseValuesAssignmentActionProcessor = void 0;
const Joi = require("joi");
const typedi_1 = require("typedi");
const js_yaml_1 = require("js-yaml");
const models_1 = require("../../models");
const utils_1 = require("../../utils");
const services_1 = require("../../services");
const errors_1 = require("../../errors");
class BaseValuesAssignmentActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return BaseValuesAssignmentActionProcessor.validationSchema;
    }
    /**
     * Get value assignment target based on assignment key
     * @param context
     * @param parameters
     */
    getAssignmentTarget() {
        const key = this.getAssignmentKey();
        return this.context[key];
    }
    /**
     * @inheritdoc
     */
    validate() {
        const _super = Object.create(null, {
            validate: { get: () => super.validate }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.validate.call(this);
            if (this.options.$ && this.options.$.inline) {
                try {
                    Joi.assert(this.options.$.inline, Joi.object().required());
                }
                catch (e) {
                    throw new errors_1.ActionError(e.details.map((d) => d.message).join('\n'), errors_1.INVALID_CONFIGURATION);
                }
            }
        });
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const target = this.getAssignmentTarget();
            const flowService = typedi_1.Container.get(services_1.FlowService);
            const names = Object.keys(this.options);
            const promises = names.map((name) => __awaiter(this, void 0, void 0, function* () {
                const option = this.options[name];
                let override = option.override;
                const children = option.children;
                const priorityOnFiles = option.priority === 'files';
                if (option.files) {
                    if ((option.inline !== undefined || option.inline === null) && priorityOnFiles) {
                        if (option.push) {
                            utils_1.ContextUtil.push(target, name, option.inline, children, override);
                        }
                        else {
                            utils_1.ContextUtil.assign(target, name, option.inline, override);
                        }
                        override = false;
                    }
                    const files = yield utils_1.FSUtil.findFilesByMasks(option.files, [], this.snapshot.wd);
                    for (const path of files) {
                        this.snapshot.log(`Reading from file: ${path} for key ${name}`);
                        let fileContent = yield utils_1.FSUtil.readTextFile(path);
                        // resolve global template
                        fileContent = yield flowService.resolveTemplate(this.context.ejsTemplateDelimiters.global, fileContent, this.context, this.snapshot, this.parameters);
                        let fileContentObject = js_yaml_1.safeLoad(fileContent);
                        // resolve local template
                        fileContentObject = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, fileContentObject, this.context, this.snapshot, this.parameters, false);
                        // resolve references
                        fileContentObject = utils_1.ContextUtil.resolveReferences(fileContentObject, this.context, this.parameters);
                        if (option.push) {
                            utils_1.ContextUtil.push(target, name, fileContentObject, children, override);
                        }
                        else {
                            utils_1.ContextUtil.assign(target, name, fileContentObject, override);
                        }
                        override = false;
                    }
                }
                if ((option.inline !== undefined || option.inline === null) && !priorityOnFiles) {
                    if (option.push) {
                        utils_1.ContextUtil.push(target, name, option.inline, children, override);
                    }
                    else {
                        utils_1.ContextUtil.assign(target, name, option.inline, override);
                    }
                }
            }));
            yield Promise.all(promises);
        });
    }
}
exports.BaseValuesAssignmentActionProcessor = BaseValuesAssignmentActionProcessor;
BaseValuesAssignmentActionProcessor.validationSchema = Joi.object()
    .pattern(/^\$(\.[^.]+)*$/, Joi.object({
    inline: Joi.any(),
    files: Joi.array().items(Joi.string()).min(1),
    priority: Joi.string().valid('inline', 'files'),
    override: Joi.boolean(),
    push: Joi.boolean(),
    children: Joi.boolean(),
})
    .required()
    .or('inline', 'files')
    .unknown(false))
    .min(1)
    .required()
    .options({ abortEarly: true });
//# sourceMappingURL=BaseValuesAssignmentActionProcessor.js.map