"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VirtualFlowActionHandler = exports.VirtualFlowActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const services_1 = require("../../services");
const typedi_1 = require("typedi");
const jsonschema_1 = require("jsonschema");
const schemas_1 = require("../../schemas");
const object_collider_1 = require("object-collider");
const errors_1 = require("../../errors");
class VirtualFlowActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return VirtualFlowActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    validate() {
        const _super = Object.create(null, {
            validate: { get: () => super.validate }
        });
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            let unmaskedParametersSchema;
            if (this.options.hasOwnProperty('parametersSchema')) {
                const masked = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.parametersSchema, this.context, this.snapshot, this.parameters, true);
                unmaskedParametersSchema = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.parametersSchema, this.context, this.snapshot, this.parameters, false);
                this.options.parametersSchema = masked;
            }
            let unmaskedDefaults;
            if (this.options.hasOwnProperty('defaults')) {
                const masked = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.defaults, this.context, this.snapshot, this.parameters, true);
                unmaskedDefaults = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.defaults, this.context, this.snapshot, this.parameters, false);
                this.options.defaults = masked;
            }
            this.snapshot.setOptions(this.options);
            this.options.parametersSchema = unmaskedParametersSchema;
            this.options.defaults = unmaskedDefaults;
            yield _super.validate.call(this);
        });
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let virtualDefaults;
            if (this.options.defaults) {
                virtualDefaults = {
                    values: this.options.defaults.values,
                };
                if (this.options.defaults.mergeFunction) {
                    const script = [
                        'return function(defaults, parameters) {',
                        this.options.defaults.mergeFunction,
                        '}',
                    ].join('\n');
                    virtualDefaults.mergeFunction = new Function(script)();
                }
                if (this.options.defaults.modifiers) {
                    virtualDefaults.modifiers = {};
                    for (const path of Object.keys(this.options.defaults.modifiers)) {
                        const script = [
                            'return function(defaults, parameters) {',
                            this.options.defaults.modifiers[path],
                            '}',
                        ].join('\n');
                        virtualDefaults.modifiers[path] = new Function(script)();
                    }
                }
            }
            const dynamicFlowHandler = new DynamicFlowActionHandler(this.options.dynamicWorkDir ? null : this.snapshot.wd, this.snapshot.wd, this.options.id, this.options.aliases || [], this.options.parametersSchema, this.options.action, virtualDefaults);
            this.context.dynamicActionHandlers.register(dynamicFlowHandler, require('./index'), this.snapshot);
        });
    }
}
exports.VirtualFlowActionProcessor = VirtualFlowActionProcessor;
VirtualFlowActionProcessor.validationSchema = Joi.object({
    id: Joi.string().min(1).required(),
    aliases: Joi.array().items(Joi.string().min(1)),
    defaults: Joi.object({
        values: Joi.any().required(),
        mergeFunction: Joi.string().min(1),
        modifiers: Joi.object().pattern(/\$(\.[^\.]+)*/, Joi.string()),
    })
        .without('mergeFunction', 'modifiers')
        .without('modifiers', 'mergeFunction'),
    parametersSchema: Joi.object({
        id: Joi.string(),
        $schema: Joi.string(),
        $ref: Joi.string(),
        title: Joi.string(),
        description: Joi.string(),
        multipleOf: Joi.number(),
        maximum: Joi.number(),
        exclusiveMaximum: Joi.boolean(),
        minimum: Joi.number(),
        exclusiveMinimum: Joi.boolean(),
        maxLength: Joi.number(),
        minLength: Joi.number(),
        pattern: Joi.string(),
        additionalItems: Joi.alternatives(Joi.boolean(), Joi.link('#schema')),
        items: Joi.alternatives(Joi.link('#schema'), Joi.array().items(Joi.link('#schema'))),
        maxItems: Joi.number(),
        minItems: Joi.number(),
        uniqueItems: Joi.boolean(),
        maxProperties: Joi.number(),
        minProperties: Joi.number(),
        required: Joi.array().items(Joi.string()),
        additionalProperties: Joi.alternatives(Joi.boolean(), Joi.link('#schema')),
        definitions: Joi.object().pattern(/^/, Joi.link('#schema')),
        properties: Joi.object().pattern(/^/, Joi.link('#schema')),
        patternProperties: Joi.object().pattern(/^/, Joi.link('#schema')),
        dependencies: Joi.object().pattern(/^/, Joi.alternatives(Joi.link('#schema'), Joi.array().items(Joi.string()))),
        enum: Joi.array().items(Joi.any()),
        type: Joi.alternatives(Joi.string(), Joi.array().items(Joi.string())),
        format: Joi.string(),
        allOf: Joi.array().items(Joi.link('#schema')),
        anyOf: Joi.array().items(Joi.link('#schema')),
        oneOf: Joi.array().items(Joi.link('#schema')),
        not: Joi.link('#schema'),
    }).id('schema'),
    dynamicWorkDir: Joi.boolean(),
    action: schemas_1.FBL_ACTION_SCHEMA,
}).required();
class VirtualFlowActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return VirtualFlowActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new VirtualFlowActionProcessor(options, context, snapshot, parameters);
    }
}
exports.VirtualFlowActionHandler = VirtualFlowActionHandler;
VirtualFlowActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.virtual',
    aliases: ['fbl.flow.virtual', 'flow.virtual', 'virtual'],
    skipTemplateProcessing: true,
};
class DynamicFlowActionProcessor extends models_1.ActionProcessor {
    constructor(options, context, snapshot, parameters, wd, pwd, validationSchema, action, virtualDefaults) {
        super(options, context, snapshot, parameters);
        this.wd = wd;
        this.pwd = pwd;
        this.validationSchema = validationSchema;
        this.action = action;
        this.virtualDefaults = virtualDefaults;
    }
    /**
     * @inheritdoc
     */
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.validationSchema) {
                const mergedOptions = this.getMergedOptions(this.options);
                const result = new jsonschema_1.Validator().validate(mergedOptions, this.validationSchema);
                if (!result.valid) {
                    throw new errors_1.ActionError(result.errors
                        .map((e) => {
                        let key = 'value';
                        /* istanbul ignore else */
                        if (e.property !== 'instance') {
                            key = e.property.substring('instance.'.length);
                        }
                        return `${key} ${e.message}`;
                    })
                        .join('\n'), errors_1.INVALID_CONFIGURATION);
                }
            }
        });
    }
    /**
     * Merge passed options with default values
     * @param options
     */
    getMergedOptions(options) {
        let mergedOptions = options;
        if (this.virtualDefaults) {
            if (this.virtualDefaults.mergeFunction) {
                const clonedDefaults = JSON.parse(JSON.stringify(this.virtualDefaults.values));
                mergedOptions = this.virtualDefaults.mergeFunction(clonedDefaults, options);
            }
            else {
                mergedOptions = object_collider_1.collide(this.virtualDefaults.values, options, this.virtualDefaults.modifiers);
            }
        }
        return mergedOptions;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            if (this.wd) {
                this.snapshot.wd = this.wd;
            }
            this.parameters.parameters = this.getMergedOptions(this.options);
            this.parameters.parameters.wd = this.snapshot.wd;
            this.parameters.parameters.pwd = this.pwd;
            const childSnapshot = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, this.action, this.context, this.parameters, this.snapshot);
            this.snapshot.registerChildActionSnapshot(childSnapshot);
        });
    }
}
/**
 * Dynamic flow handler that is being created dynamically by VirtualFlowHandler
 */
class DynamicFlowActionHandler extends models_1.ActionHandler {
    constructor(wd, pwd, id, aliases, validationSchema, action, virtualDefaults) {
        super();
        this.wd = wd;
        this.pwd = pwd;
        this.id = id;
        this.aliases = aliases;
        this.validationSchema = validationSchema;
        this.action = action;
        this.virtualDefaults = virtualDefaults;
    }
    /**
     * @inheritdoc
     */
    getWorkingDirectory() {
        return this.wd;
    }
    /**
     * @inheritdoc
     */
    getMetadata() {
        return {
            id: this.id,
            aliases: this.aliases,
        };
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new DynamicFlowActionProcessor(options, context, snapshot, parameters, this.wd, this.pwd, this.validationSchema, this.action, this.virtualDefaults);
    }
}
//# sourceMappingURL=VirtualFlowActionHandler.js.map